if not modules then modules = { } end modules ['t-annotation'] = {
    version   = 1.000,
    comment   = "Annotations",
    author    = "Wolfgang Schuster",
    copyright = "Wolfgang Schuster",
    license   = "GNU General Public License"
}

thirddata            = thirddata            or {}
thirddata.annotation = thirddata.annotation or {}


local annotation   = thirddata.annotation

local variables    = interfaces.variables

local v_yes        = variables.yes
local v_no         = variables.no
local v_auto       = variables.auto
local v_paragraph  = variables.paragraph
local v_text       = variables.text
local v_command    = variables.command
local v_vertical   = variables.vertical
local v_horizontal = variables.horizontal

-- Process the values of the environment

function annotation.save(t)
    local alternative = t.alternative
    local display     = t.display
    local paragraph   = t.paragraph
    local indenting   = t.indenting
    local alignment   = t.alignment
    local mode        = t.mode
    local export      = t.export
    local title       = t.title
    local list        = t.list
    local number      = t.number
    local text        = t.text
    context.unprotect()
    -- alternative
    if alternative == v_paragraph then
        tex.settrue('annotation_conditional_display')
        tex.settrue('annotation_conditional_paragraph')
    elseif alternative == v_text then
        tex.settrue('annotation_conditional_inline')
    elseif alternative == v_command then
        -- display/inline mode
        if display == v_yes then
            tex.settrue('annotation_conditional_display')
        elseif display == v_no then
            tex.settrue('annotation_conditional_inline')
        else
            if mode == v_vertical then
                tex.settrue('annotation_conditional_display')
            else
                tex.settrue('annotation_conditional_inline')
            end
        end
        -- paragraph mode
        if paragraph == v_yes then
            tex.settrue('annotation_conditional_paragraph')
        end
    elseif mode == v_horizontal then
        tex.settrue('annotation_conditional_inline')
    end
    -- number
    if number == v_yes then
        tex.settrue('annotation_conditional_number')
    else
        tex.setfalse('annotation_conditional_number')
    end
    -- format
    if alternative == v_paragraph or alternative == v_command and tex.conditionals['annotation_conditional_paragraph'] then
        if indenting ~= "" then context.setupindenting{indenting} end
        if alignment ~= "" then context.setupalign    {alignment} end
    end
    -- title
    if title ~= "" then
        tex.settrue('annotation_conditional_title')
    end
    -- list
    if list ~= "" then
        tex.settrue('annotation_conditional_caption')
    end
    -- text
    if tex.conditionals['annotation_conditional_header'] and text == "" then
        tex.setfalse('annotation_conditional_header')
    end
    -- export
    --~ if export == v_yes then
    --~     annotation.write(title,content)
    --~ end
    context.protect()
end

-- Collect the content of the environment

annotation.data = { }

function annotation.erasedata()
    annotation.data = nil
end

function annotation.getdata()
    return annotation.data or ""
end

function annotation.collectdata(content,begintag,endtag)
    local data    = annotation.getdata()
    local nesting = false
    if data == "" then
        annotation.data = content
    else
        annotation.data = data .. "\\" .. endtag .. " " .. content
    end
    if select(2,string.gsub(annotation.data,begintag,begintag)) > select(2,string.gsub(annotation.data,endtag,endtag)) then
        nesting = true
    else
        nesting = false
    end
    if not nesting then annotation.data = annotation.data:strip() end
    commands.testcase(nesting)
end

-- Write the content of the environment to a file

annotation.empty = true

function annotation.open()
    annotation.export = io.open(file.addsuffix(table.concat({tex.jobname,"annotation"},"-"),"txt"),"wb")
end

function annotation.write(title,content)
    local title, content = title:strip(), content
    if annotation.empty == true then
        annotation.open()
        annotation.empty = false
    else
        annotation.export:write("\n\n")
    end
    if title ~= "" then
        annotation.export:write(title)
        annotation.export:write("\n",string.rep("-",#title),"\n\n")
        content = content:gsub(" \\par ","\n\n")
        annotation.export:write(content)
    end
end

--~ function annotation.write(title,content)
--~     local title, content = title:strip(), content
--~     if annotation.empty == true then
--~         annotation.open()
--~         annotation.empty = false
--~     else
--~         annotation.export:write("\n")
--~     end
--~     annotation.export:write(title.." "..content)
--~ end
